/* 
   raw2qap.c  
   Program to convert species abundances to matrices to be used with QAP.    
   Reads an n by m data matrix and produces distance and flow matrices.
   Raw data start with #rows, #columns, followed by data.
   Species are columns, samples are rows.
   Language: ISO C.
   Input: number of rows, number of columns, data matrix rowwise.
   Output: distance and flow matrix formatted for QAP input.
   Depends on: libmatrix
   License: GPL <http://www.gnu.org/copyleft/gpl.html>
*/


#include <u.h>
#include <libc.h>
#include <bio.h>
#include <stdio.h>
//#include <ctype.h>
#include <libmatrix.h>
#include <header.h>
#include <imatrix.h>
#include <matrix.h>

Biobuf bin;


/* multiply flows by NUM/maxflow */
#define NUM 1000.0	 



void
main(int argc, char *argv[])
{
  int i, j, k, rows, cols, x, maxflow = 0, num, big = 0, z, f=0;
  iMatrix  *matrix;    /* data matrix (int) */


  /* read-in command-line params */
  num = NUM;
  if (argc > 1)
   big  = atoi(argv[1]);
  if (argc > 2)
    num = atoi(argv[2]);

	Binit(&bin, f, OREAD);

  /* read-in number of species, number of samples */
	//read header
	mat_readheader(&rows, &cols);
//print ("rows cols:%d %d\n", rows, cols);

  /* alloc the matrix */
	//allocate memory
	matrix=imat_alloc(rows, cols);
  /* read-in the matrix */

	// read matrix in raw form
	imat_read(matrix);




  /* print dimension */
      print("%d \n ", rows);

  /* calculate distances between sequence's equidistant nodes */
  /* nodes are assumed equidistant since we have no info about real distances */
  for (i = 0; i < rows; i++){
    for (j = 0; j < rows; j++){
      printf("%d ", abs(i - j));
    } 
    printf("\n");
  }
  printf("\n");

  if (big) {
    
    /* calculate max flow */
    maxflow = 0;
    for (i = 0; i < rows; i++){
      for (j = 0; j < rows; j++){
	x = 0;	
	for (k = 0; k < cols; k++){
	  /* x += matrix[i][k] * matrix[j][k]; */
	  x += matrix->x[i][k] * matrix->x[j][k];
	}
	if(x > maxflow) maxflow = x;
      }  
    }
  }

  /* calculate flows and print them out*/

  for (i = 0; i < rows; i++){
    for (j = 0; j < rows; j++){
      x = 0;	
      for (k = 0; k < cols; k++){
	/* x += matrix[i][k] * matrix[j][k]; */
	x += matrix->x[i][k] * matrix->x[j][k];
      }    
      print("%d ", big ? (num*x/maxflow): x);
    }  
    print("\n");
  }
  printf("\n");
  exits(0);
}
/* eof */
